
# TODO rconcile requiremtnes
# todo implement screen stuff
# todo alerts
# todo scan for request emails

import time
from controller import Controller, EnvError, SensorError
from screen import Screen

# frequency of sensor update in seconds
UPDATE_FREQ = 1


def main(controller, screen=None):
    x = 2
    while x > 1:
        try:
            time.sleep(UPDATE_FREQ)
            controller.update_controller()
            controller.update_data_records(to_console=True)
        except SensorError:
            print("Sensor disconnected")
            # todo send sensor disconnected message
            pass

        except EnvError:
            print("Environment Issue!")
            # todo send env alert message
            pass

        except OSError:
            print('Sensor disconnected!')
            # todo send env alert message
            pass

        try:
            lines = ['T1: {}C, T2: {}C'.format(round(controller.T1, 1), round(controller.T2, 1)),
                     'H1: {}%, H2: {}%'.format(round(controller.H1, 1), round(controller.H2, 1)),
                     'Battery: {}/{}'.format(controller.batt_charge, controller.batt_capacity)]
            screen.display(lines=lines)

        except ValueError:
            print("Screen disconnected")
            # todo send screen disconnected message
            pass


if __name__ == '__main__':
    try:
        controller = Controller()
        try:
            screen = Screen()
            main(controller, screen)

        except ValueError:
            main(controller)
            print("Screen disconnected")
            # todo send screen disconnected message
            pass

    except SensorError:
        print("Sensor disconnected")
        # todo send sensor disconnected message
        pass

    except EnvError:
        print("Environment Issue!")
        # todo send env alert message
        pass






from __future__ import print_function

import os.path

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

from email.mime.text import MIMEText
import base64

# If modifying these scopes, delete the file token.json.
# SCOPES = ['https://www.googleapis.com/auth/gmail.readonly']
SCOPES = ['https://www.googleapis.com/auth/gmail.send', 'https://mail.google.com/', 'https://www.googleapis.com/auth/gmail.modify', 'https://www.googleapis.com/auth/gmail.compose']
# SCOPES = ['https://www.googleapis.com/auth/gmail.send']


class GMailAcc:
    def __init__(self):
        self.creds = None
        self.address = 'ysr.monitor@gmail.com'
        # The file token.json stores the user's access and refresh tokens, and is
        # created automatically when the authorization flow completes for the first
        # time.
        if os.path.exists('token.json'):
            self.creds = Credentials.from_authorized_user_file('token.json', SCOPES)
        # If there are no (valid) credentials available, let the user log in.
        if not self.creds or not self.creds.valid:
            if self.creds and self.creds.expired and self.creds.refresh_token:
                self.creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    'credentials.json', SCOPES)
                self.creds = flow.run_local_server(port=0)
            # Save the credentials for the next run
            with open('token.json', 'w') as token:
                token.write(self.creds.to_json())

        self.service = build('gmail', 'v1', credentials=self.creds)

    def get_labels(self):
        try:
            # Call the Gmail API
            results = self.service.users().labels().list(userId='me').execute()
            labels = results.get('labels', [])

            if not labels:
                print('No labels found.')
                return
            print('Labels:')
            for label in labels:
                print(label['name'])

        except HttpError as error:
            # TODO(developer) - Handle errors from gmail API.
            print(f'An error occurred: {error}')

    def create_message(self, sender, to, subject, message_text):
        message = MIMEText(message_text)
        message['to'] = to
        message['from'] = sender
        message['subject'] = subject
        raw_message = base64.urlsafe_b64encode(message.as_string().encode("utf-8"))

        return {'raw': raw_message.decode("utf-8")}

    def send_message(self, user_id, message):
        try:
            message = self.service.users().messages().send(userId=user_id, body=message).execute()
            print('Message Id: %s' % message['id'])
            return message
        except Exception as e:
            print('An error occurred: %s' % e)
            return None


if __name__ == '__main__':
    acc = GMailAcc()
    print(acc.get_labels())

    YSR_EMAIL = 'ysr.monitor@gmail.com'
    new_message = acc.create_message(YSR_EMAIL, YSR_EMAIL, 'test', 'test_message')
    acc.send_message(YSR_EMAIL, new_message)


import dfr0528 as ups
import datetime
import pathlib
import subprocess
import re
import smbus2
import bme280
import tabulate
import os
import pytz
from email_handler import GMailAcc
# from alert import AlertFactory

DATA_FILE_NAME = 'data.txt'
DATA_FILEPATH = pathlib.Path(__file__).parent.resolve().joinpath(DATA_FILE_NAME)


class Controller:
    def __init__(self):
        self.alerts = list()
        self.bus_addrs = self.init_bus_vars()
        # self.check_bus()

        self.bme_sensors = self.init_temp_sensors()
        self.ups = ups.DFR0528()

        self.T1 = None
        self.T2 = None
        self.TAvg = None

        self.H1 = None
        self.H2 = None
        self.HAvg = None

        self.batt_charge = None
        self.batt_capacity = self.ups.total_capacity

    def update_status(self, interval):
        gmail_acc = GMailAcc()  # init gmail account

        # send status email with Status as subject line, and response interval in seconds with s appended
        email = gmail_acc.create_message(gmail_acc.address, gmail_acc.address, 'Status', interval + 's')
        gmail_acc.send_message(gmail_acc.address, email)

        return

    def init_temp_sensors(self):
        """ init bme sensors and BME objects -> dict containing BME objects """

        bmes = dict()
        try:
            bmes['bme1'] = self.sample_bme(self.bus_addrs['bme1'])
        except OSError:
            raise SensorError
        try:
            bmes['bme2'] = self.sample_bme(self.bus_addrs['bme2'])
        except OSError:
            raise SensorError

        return bmes

    def sample_bme(self, addr):
        port = 1
        bus = smbus2.SMBus(port)

        calibration_params = bme280.load_calibration_params(bus, addr)

        # the sample method will take a single reading and return a
        # compensated_reading object
        data = bme280.sample(bus, addr, calibration_params)
        bus.close()

        return data

    def init_bus_vars(self):
        """ init all bus addresses for expected devices"""

        bus_addrs = dict()

        bus_addrs['bme1'] = 0x76
        bus_addrs['bme2'] = 0x77
        bus_addrs['screen'] = 0x3c
        bus_addrs['ups'] = 0x10

        # self.check_bus()

        return bus_addrs

    def check_bus(self):
        """ check all requested buses are available/connected """

        p = subprocess.Popen(['i2cdetect', '-y', '1'], stdout=subprocess.PIPE,)
        match_rows = list()

        for i in range(0, 9):
            line = str(p.stdout.readline())

            for match in re.finditer("[0-9][0-9]:.*[0-9][0-9, 'a-g']", line):
                # print(match.group())
                split1 = match.group().split()
                match_rows += split1

        # remove unwanted entries from matches
        match_rows = [x for x in match_rows if '-' not in x]
        match_rows = [x for x in match_rows if ':' not in x]

        # compare to bus_addrs
        for key in self.bus_addrs.keys():
            addr = self.bus_addrs[key]
            addr = str(addr)  # only compare last two addr elements as string
            addr = addr[-2:]

            if addr not in match_rows:
                raise OSError
            else:
                print(key + " connected!")

        return

    def update_data_records(self, to_console=False):
        """ store sensing variables and timestamp """
        to_store = self.process_inputs()
        header = ['timestamp', 'temp1', 'temp2', 'avg_temp', 'hum1', 'hum2', 'avg_hum', 'batt_capacity']
        # convert header list to string for txt data
        header_str = ""
        for item in header:
            header_str += item
            header_str += ', '

        # setup comma delimited string
        x = ','.join(map(str, to_store))
        x += '\n'

        # write to txt
        with open(DATA_FILEPATH, 'a') as data_file:
            # write header to file if empty
            if os.stat(DATA_FILEPATH).st_size == 0:
                data_file.write(header_str + '\n')
            data_file.write(x)
        data_file.close()

        # debugging
        if to_console:
            print(tabulate.tabulate([to_store], header))

        return

    def update_controller(self):
        """ update state of all peripherals connected to controller """

        for bme in self.bme_sensors.keys():
            # self.temp_sensors[bme].update_sensor()
            self.bme_sensors[bme] = self.sample_bme(self.bus_addrs[bme])

        self.ups.update_capacity()

    def process_inputs(self):
        """ process sensor inputs and ensure all are within limits """

        # desired limits set by user
        temp_max_allowable = 25
        humidity_max_allowable = 80

        timestamp = datetime.datetime.now()
        timestamp = timestamp.replace(tzinfo=pytz.timezone('Canada/Yukon'))

        # data processing
        self.T1 = self.bme_sensors['bme1'].temperature
        self.T2 = self.bme_sensors['bme2'].temperature
        self.TAvg = (self.T1 + self.T2)/2

        self.H1 = self.bme_sensors['bme1'].humidity
        self.H2 = self.bme_sensors['bme2'].humidity
        self.HAvg = (self.H1 + self.H2)/2

        self.batt_charge = self.ups.charged_capacity
        # todo determine capacity and raise batterror when low

        if self.TAvg >= temp_max_allowable:
            raise EnvError
        if self.HAvg >= humidity_max_allowable:
            raise EnvError

        # STORAGE HEADER - timestamp, temp1, temp2, avg_temp, hum1, hum2, avg_hum, batt_capacity
        to_store = [timestamp, self.T1, self.T2, self.TAvg, self.H1, self.H2, self.HAvg, (self.batt_charge/self.batt_capacity)*100]

        return to_store


class EnvError(Exception):
    """ issue with one or more env variables """
    pass


class SensorError(Exception):
    """ issue with one or more env variables """
    pass


import datetime
import pytz
from email_handler import GMailAcc


def main():
    print(pytz.all_timezones)

    dt = datetime.datetime.now()
    dt = dt.replace(tzinfo=pytz.timezone('Canada/Yukon'))
    print(dt)


def test():
    gmail_acc = GMailAcc()  # init gmail account

    # send status email with Status as subject line, and response interval in seconds with s appended
    email = gmail_acc.create_message(gmail_acc.address, gmail_acc.address, 'Status', str(3600))
    gmail_acc.send_message(gmail_acc.address, email)

    return


if __name__ == '__main__':
    test()